//******************************************************************************
//  Texas Instruments Inc.
//  For DRV8860 LED driver reference design
//	MCU: MSP430G2302
//  May 2014
//  Built with CCS 5.5
//******************************************************************************

#include <msp430G2302.h>

#define EN_LOW  (P1OUT &=~BIT0)
#define EN_HIGH (P1OUT |= BIT0)
#define LATCH_HIGH (P1OUT |= BIT4)
#define LATCH_LOW (P1OUT &= ~BIT4)
#define CLK_HIGH (P1OUT |= BIT5)
#define CLK_LOW (P1OUT &= ~BIT5)
#define DIN_HIGH (P1OUT |= BIT6)
#define DIN_LOW (P1OUT &= ~BIT6)

#define LEDa BIT0
#define LEDb BIT1
#define LEDc BIT2
#define LEDd BIT3
#define LEDe BIT4
#define LEDf BIT5
#define LEDg BIT6
#define LEDpd BIT7

//////////////////
//     a
//   f   b
//     g
//   e   c
//     d    pd
//////////////////

#define D_0 (LEDa+LEDb+LEDc+LEDd+LEDe+LEDf)
#define D_1 (LEDb+LEDc)
#define D_2 (LEDa+LEDb+LEDg+LEDd+LEDe)
#define D_3 (LEDa+LEDb+LEDc+LEDd+LEDg)
#define D_4 (LEDb+LEDc+LEDg+LEDf)
#define D_5 (LEDa+LEDc+LEDd+LEDg+LEDf)
#define D_6 (LEDa+LEDc+LEDd+LEDe+LEDg+LEDf)
#define D_7 (LEDa+LEDb+LEDc)
#define D_8 (LEDa+LEDb+LEDc+LEDd+LEDe+LEDg+LEDf)
#define D_9 (LEDa+LEDb+LEDc+LEDd+LEDg+LEDf)
#define D_off (0)

#define D_a (LEDa+LEDb+LEDc+LEDe+LEDg+LEDf)
#define D_b (LEDc+LEDd+LEDe+LEDg+LEDf)
#define D_c (LEDa+LEDf+LEDd+LEDe)
#define D_d (LEDb+LEDc+LEDd+LEDe+LEDg)
#define D_e (LEDa+LEDd+LEDe+LEDg+LEDf)
#define D_f (LEDa+LEDe+LEDg+LEDf)
#define D_g (LEDa+LEDc+LEDd+LEDe+LEDf)
#define D_h (LEDb+LEDc+LEDe+LEDg+LEDf)

#define D_l (LEDf+LEDd+LEDe)
#define D_o (LEDa+LEDb+LEDc+LEDd+LEDe+LEDf)

#define D_s (LEDa+LEDc+LEDd+LEDg+LEDf)
#define D_y (LEDb+LEDc+LEDg+LEDf+LEDd)
#define D_u (LEDb+LEDc+LEDd+LEDe+LEDf)

typedef unsigned char u8;

u8 disp_state = 0;
u8 key_s_down = 0;
u8 com_counter = 0;
u8 seg_data[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
u8 com_data = 0;
u8 flag_dark = 0;
u8 flag_breathing = 0;
u8 flag_shifting = 0;
u8 flag_dotflash = 0;
u8 flag_loaded = 0;
u8 disp_number_start = 0;
u8 dim = 1;

unsigned int counter_5ms = 0;
unsigned int counter_100ms = 0;
unsigned int counter_200ms = 0;
unsigned int counter_breath = 0;
unsigned int disp_number = 0;
//u8 flag = 0;

void Send_DATA8(u8 Data); // send 8 bits data using GPIO
//void Send_SPI8(u8 Data); // send 8 bits data using SPI
void Delay_us(); // determine the serial clk frequency, include the special command
void commandSpecial(u8 s1, u8 s2, u8 s3, u8 s4); // sent special command

/*
 * main.c
 */
int main(void) {
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer

	u8 i;

	//Set Main Clk 16M
	if (CALBC1_16MHZ == 0xFF)				// If calibration constant erased
			{
		while (1)
			;                             // do not load, trap CPU!!
	}
	DCOCTL = 0;                          // Select lowest DCOx and MODx settings
	BCSCTL1 = CALBC1_16MHZ;                 // Set range
	DCOCTL = CALDCO_16MHZ;                  // Set DCO step + modulation*/

	for (i = 0; i < 200; i++)
		Delay_us();

	P1DIR |= BIT0 + BIT4 + BIT5 + BIT6;      // output
	P1OUT &= ~(BIT0 + BIT4 + BIT5 + BIT6);   //
	P1DIR &= ~(BIT1 + BIT2 + BIT3 + BIT7); // input

	P1REN |= BIT1 + BIT2 + BIT3 + BIT7;     // pull up or down
	P1OUT |= BIT1 + BIT2 + BIT3 + BIT7;     // pull up

	CCTL0 = CCIE;                             // CCR0 interrupt enabled
	CCR0 = 16000;                              // 1ms
	TACTL = TASSEL_2 + MC_2;                  // SMCLK, contmode

	EN_HIGH;

	_EINT();

	for (;;) {
		switch (disp_state) {
		case 0:  // display "HELLO   "
			flag_breathing = 0;
			flag_loaded = 0;

			seg_data[0] = D_off;
			seg_data[1] = D_h;
			seg_data[2] = D_e;
			seg_data[3] = D_l;
			seg_data[4] = D_l;
			seg_data[5] = D_o;
			seg_data[6] = D_off;
			seg_data[7] = D_off;
			break;
		case 1:  // display "HELLO" shifting
			flag_breathing = 0;

			if (flag_shifting == 1) {
				flag_shifting = 0;

				for (i = 8; i > 0; i--) {
					seg_data[i] = seg_data[i - 1];
				}
				seg_data[0] = seg_data[8];
			}

			break;
		case 2:  // display "abcdefgh" with breathing (PWM control)
			flag_breathing = 1;

			seg_data[0] = D_a;
			seg_data[1] = D_b;
			seg_data[2] = D_c;
			seg_data[3] = D_d;
			seg_data[4] = D_e;
			seg_data[5] = D_f;
			seg_data[6] = D_g;
			seg_data[7] = D_h;

			break;
		case 3:   // display "05.23.2014"
			flag_breathing = 0;

			seg_data[0] = D_0;
			seg_data[1] = D_5 + LEDpd;
			seg_data[2] = D_2;
			seg_data[3] = D_0 + LEDpd;
			seg_data[4] = D_2;
			seg_data[5] = D_0;
			seg_data[6] = D_1;
			seg_data[7] = D_4;

			break;
		case 4: // display 0,1,2,3... every 0.1s ++
			flag_breathing = 0;
			disp_number_start = 1;

			if (disp_number == 0)  // turn off the numbers at start
					{
				seg_data[0] = D_off;
				seg_data[1] = D_off;
				seg_data[2] = D_off;
				seg_data[3] = D_off;
				seg_data[4] = D_off;
				seg_data[5] = D_off;
				seg_data[6] = D_off;
				seg_data[7] = D_off;
			}

			switch (disp_number % 10) {
			case 0:
				seg_data[7] = D_0;
				break;
			case 1:
				seg_data[7] = D_1;
				break;
			case 2:
				seg_data[7] = D_2;
				break;
			case 3:
				seg_data[7] = D_3;
				break;
			case 4:
				seg_data[7] = D_4;
				break;
			case 5:
				seg_data[7] = D_5;
				break;
			case 6:
				seg_data[7] = D_6;
				break;
			case 7:
				seg_data[7] = D_7;
				break;
			case 8:
				seg_data[7] = D_8;
				break;
			case 9:
				seg_data[7] = D_9;
				break;
			default:
				break;
			}

			if (disp_number >= 10) {
				switch (disp_number / (10) % 10) {
				case 0:
					seg_data[6] = D_0;
					break;
				case 1:
					seg_data[6] = D_1;
					break;
				case 2:
					seg_data[6] = D_2;
					break;
				case 3:
					seg_data[6] = D_3;
					break;
				case 4:
					seg_data[6] = D_4;
					break;
				case 5:
					seg_data[6] = D_5;
					break;
				case 6:
					seg_data[6] = D_6;
					break;
				case 7:
					seg_data[6] = D_7;
					break;
				case 8:
					seg_data[6] = D_8;
					break;
				case 9:
					seg_data[6] = D_9;
					break;
				default:
					break;
				}
			}

			if (disp_number >= 100) {
				switch (disp_number / (100) % 10) {
				case 0:
					seg_data[5] = D_0;
					break;
				case 1:
					seg_data[5] = D_1;
					break;
				case 2:
					seg_data[5] = D_2;
					break;
				case 3:
					seg_data[5] = D_3;
					break;
				case 4:
					seg_data[5] = D_4;
					break;
				case 5:
					seg_data[5] = D_5;
					break;
				case 6:
					seg_data[5] = D_6;
					break;
				case 7:
					seg_data[5] = D_7;
					break;
				case 8:
					seg_data[5] = D_8;
					break;
				case 9:
					seg_data[5] = D_9;
					break;
				default:
					break;
				}
			}

			if (disp_number >= 1000) {
				switch (disp_number / (1000) % 10) {
				case 0:
					seg_data[4] = D_0;
					break;
				case 1:
					seg_data[4] = D_1;
					break;
				case 2:
					seg_data[4] = D_2;
					break;
				case 3:
					seg_data[4] = D_3;
					break;
				case 4:
					seg_data[4] = D_4;
					break;
				case 5:
					seg_data[4] = D_5;
					break;
				case 6:
					seg_data[4] = D_6;
					break;
				case 7:
					seg_data[4] = D_7;
					break;
				case 8:
					seg_data[4] = D_8;
					break;
				case 9:
					seg_data[4] = D_9;
					break;
				default:
					break;
				}
			}

			if (disp_number >= 10000) {
				switch (disp_number / (10000) % 10) {
				case 0:
					seg_data[3] = D_0;
					break;
				case 1:
					seg_data[3] = D_1;
					break;
				case 2:
					seg_data[3] = D_2;
					break;
				case 3:
					seg_data[3] = D_3;
					break;
				case 4:
					seg_data[3] = D_4;
					break;
				case 5:
					seg_data[3] = D_5;
					break;
				case 6:
					seg_data[3] = D_6;
					break;
				case 7:
					seg_data[3] = D_7;
					break;
				case 8:
					seg_data[3] = D_8;
					break;
				case 9:
					seg_data[3] = D_9;
					break;
				default:
					break;
				}
			}

			break;
		case 5:   // display "see you " with breathing and shifting
			disp_number_start = 0; // stop the counter for case 5
			flag_breathing = 1;

			if (flag_loaded == 0) {
				seg_data[0] = D_s;
				seg_data[1] = D_e;
				seg_data[2] = D_e;
				seg_data[3] = D_off;
				seg_data[4] = D_y;
				seg_data[5] = D_o;
				seg_data[6] = D_u;
				seg_data[7] = D_off;

				flag_loaded = 1;
			}

			if (flag_shifting == 1) {
				flag_shifting = 0;

				for (i = 8; i > 0; i--) {
					seg_data[i] = seg_data[i - 1];
				}
				seg_data[0] = seg_data[8];
			}

			break;

		default:
			break;
		}
	}

}

void Delay_us() {
	u8 n;
	for (n = 5; n > 0; n--)
		_nop();
}

void commandSpecial(u8 s1, u8 s2, u8 s3, u8 s4) {
	u8 temp;
	LATCH_HIGH;
	Delay_us();
	CLK_LOW;
	Delay_us();
	LATCH_LOW;
	Delay_us();

	for (temp = 0; temp < s1; temp++) {
		CLK_HIGH;
		Delay_us();
		CLK_LOW;
		Delay_us();
	}

	LATCH_HIGH;
	Delay_us();
	LATCH_LOW;
	Delay_us();

	for (temp = 0; temp < s2; temp++) {
		CLK_HIGH;
		Delay_us();
		CLK_LOW;
		Delay_us();
	}

	LATCH_HIGH;
	Delay_us();
	LATCH_LOW;
	Delay_us();

	for (temp = 0; temp < s3; temp++) {
		CLK_HIGH;
		Delay_us();
		CLK_LOW;
		Delay_us();
	}

	LATCH_HIGH;
	Delay_us();
	LATCH_LOW;
	Delay_us();

	for (temp = 0; temp < s4; temp++) {
		CLK_HIGH;
		Delay_us();
		CLK_LOW;
		Delay_us();
	}
}

void Send_DATA8(u8 Data) {
	u8 i;
	P1SEL &= ~(BIT5 + BIT6);

	CLK_LOW;
	for (i = 8; i != 0; i--) {
		(Data & 0x80) ? DIN_HIGH : DIN_LOW;
		Delay_us();
		CLK_HIGH;
		Data = (Data << 1);
		Delay_us();
		if (i > 1)
			CLK_LOW;  // left CLK HIGH at after the last data transfered
					  //avoid unwanted fault register pushed out.
	}
}

// Timer A0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A(void) {
	CCR0 += 16000;                            // 1ms

	///////////////////////////////////// dynamic display with COM scan

	if (com_counter == 8)
		com_counter = 0;

	LATCH_HIGH;
	CLK_LOW;
	Delay_us();
	LATCH_LOW;
	Delay_us();
	Send_DATA8(seg_data[com_counter]);                            // Seg data
	Send_DATA8(1 << (com_counter)); // COM data  scan COM with one LED lighting at one time (0.5ms)
	Delay_us();
	LATCH_HIGH;
	com_counter++;

	///////////////////////////////////////////////////////////////////////////

	counter_5ms++;
	if (counter_5ms == 5)                     // 5ms for key dealing
			{
		if (((P1IN & BIT3)==0)&&(key_s_down==0) ) // button pushed
{		key_s_down = 1;
		disp_state ++;                         // display state change
		if (disp_state == 6) disp_state = 0;
	}
	else if (P1IN & BIT3)
	{
		key_s_down = 0;
	}
	counter_5ms = 0;
}

		///////////////////// Display shifting
	counter_200ms++;
	if (counter_200ms == 200)   //0.2s
			{
		counter_200ms = 0;

		flag_shifting = 1;
	}

	////////////////////////// Display a running counter
	counter_100ms++;
	if (counter_100ms == 100)   //0.1s
			{
		counter_100ms = 0;

		if (disp_number_start == 1)
			disp_number++;
		else
			disp_number = 0;
	}

	/////////////////////////// Auto soft flashing ("breathing")
	counter_breath++;
	if (counter_breath == 70)  // PWM change every 70ms
			{
		counter_breath = 0;

		dim++;
		if (dim == 15)
			dim = 1;
		if (flag_breathing == 1) {
			if (dim <= 7) {
				commandSpecial(1, 2, 2, 3); // Ctrl Prog En
				LATCH_HIGH;
				CLK_LOW;
				Delay_us();
				LATCH_LOW;
				Send_DATA8(0x80 + (dim << 4)); // PWM to Seg
				Send_DATA8(0x00); // no PWM to COM
				Delay_us();
				LATCH_HIGH;
			} else {
				commandSpecial(1, 2, 2, 3); // Ctrl Prog En
				LATCH_HIGH;
				CLK_LOW;
				Delay_us();
				LATCH_LOW;
				Send_DATA8(0x80 + ((15 - dim) << 4)); // PWM to Seg
				Send_DATA8(0x00); // no PWM to COM
				Delay_us();
				LATCH_HIGH;
			}
		} else {
			commandSpecial(1, 2, 2, 3); // Ctrl Prog En
			LATCH_HIGH;
			CLK_LOW;
			Delay_us();
			LATCH_LOW;
			Send_DATA8(0x00); // 100%
			Send_DATA8(0x00); // 100%
			Delay_us();
			LATCH_HIGH;
		}
	}
}
